@@ -25,6 +25,7 @@ import re
 import sys
 import unicodedata
 import string
+import warnings
 
 from traitlets.config.configurable import Configurable
 from IPython.core.error import TryNext
@@ -37,11 +38,6 @@ from IPython.utils.process import arg_split
 from IPython.utils.py3compat import builtin_mod, string_types, PY3, cast_unicode_py2
 from traitlets import Bool, Enum, observe
 
-from functools import wraps
-
-#-----------------------------------------------------------------------------
-# Globals
-#-----------------------------------------------------------------------------
 
 # Public API
 __all__ = ['Completer','IPCompleter']
@@ -52,48 +48,6 @@ else:
     PROTECTABLES = ' ()[]{}?=\\|;:\'#*"^&'
 
 
-#-----------------------------------------------------------------------------
-# Work around BUG decorators.
-#-----------------------------------------------------------------------------
-
-def _strip_single_trailing_space(complete):
-    """
-    This is a workaround for a weird IPython/Prompt_toolkit behavior,
-    that can be removed once we rely on a slightly more recent prompt_toolkit
-    version (likely > 1.0.3). So this can likely be removed in IPython 6.0
-
-    cf https://github.com/ipython/ipython/issues/9658
-    and https://github.com/jonathanslenders/python-prompt-toolkit/pull/328
-
-    The bug is due to the fact that in PTK the completer will reinvoke itself
-    after trying to completer to the longuest common prefix of all the
-    completions, unless only one completion is available.
-
-    This logic is faulty if the completion ends with space, which can happen in
-    case like::
-
-        from foo import im<ta>
-
-    which only matching completion is `import `. Note the leading space at the
-    end. So leaving a space at the end is a reasonable request, but for now
-    we'll strip it.
-    """
-
-    @wraps(complete)
-    def comp(*args, **kwargs):
-        text, matches =  complete(*args, **kwargs)
-        if len(matches) == 1:
-            return text, [matches[0].rstrip()]
-        return text, matches
-
-    return comp
-
-
-
-#-----------------------------------------------------------------------------
-# Main functions and classes
-#-----------------------------------------------------------------------------
-
 def has_open_quotes(s):
     """Return whether a string has open quotes.
 
@@ -588,6 +542,13 @@ class IPCompleter(Completer):
         """,
     ).tag(config=True)
 
+    @observe('limit_to__all__')
+    def _limit_to_all_changed(self, change):
+        warnings.warn('`IPython.core.IPCompleter.limit_to__all__` configuration '
+            'value has been deprecated since IPython 5.0, will be made to have '
+            'no effects and then removed in future version of IPython.',
+            UserWarning)
+
     def __init__(self, shell=None, namespace=None, global_namespace=None,
                  use_readline=True, config=None, **kwargs):
         """IPCompleter() -> completer
@@ -1128,7 +1089,6 @@ class IPCompleter(Completer):
 
         return None
 
-    @_strip_single_trailing_space
     def complete(self, text=None, line_buffer=None, cursor_pos=None):
         """Find completions for the given text and line context.
 
@@ -1164,6 +1124,9 @@ class IPCompleter(Completer):
         if cursor_pos is None:
             cursor_pos = len(line_buffer) if text is None else len(text)
 
+        if self.use_main_ns:
+            self.namespace = __main__.__dict__
+
         if PY3:
 
             base_text = text if not line_buffer else line_buffer[:cursor_pos]
