@@ -13,7 +13,8 @@ The code in this file is mainly lifted out of cmd.py in Python 2.2, with minor
 changes. Licensing should therefore be under the standard Python terms.  For
 details on the PSF (Python Software Foundation) standard license, see:
 
-http://www.python.org/2.2.3/license.html"""
+https://docs.python.org/2/license.html
+"""
 
 #*****************************************************************************
 #
@@ -64,7 +65,7 @@ def BdbQuit_excepthook(et, ev, tb, excepthook=None):
     parameter.
     """
     warnings.warn("`BdbQuit_excepthook` is deprecated since version 5.1",
-                  DeprecationWarning)
+                  DeprecationWarning, stacklevel=2)
     if et==bdb.BdbQuit:
         print('Exiting Debugger.')
     elif excepthook is not None:
@@ -77,7 +78,7 @@ def BdbQuit_excepthook(et, ev, tb, excepthook=None):
 def BdbQuit_IPython_excepthook(self,et,ev,tb,tb_offset=None):
     warnings.warn(
         "`BdbQuit_IPython_excepthook` is deprecated since version 5.1",
-        DeprecationWarning)
+        DeprecationWarning, stacklevel=2)
     print('Exiting Debugger.')
 
 
@@ -129,7 +130,7 @@ class Tracer(object):
         """
         warnings.warn("`Tracer` is deprecated since version 5.1, directly use "
                       "`IPython.core.debugger.Pdb.set_trace()`",
-                      DeprecationWarning)
+                      DeprecationWarning, stacklevel=2)
 
         ip = get_ipython()
         if ip is None:
@@ -202,7 +203,7 @@ def _file_lines(fname):
         return out
 
 
-class Pdb(OldPdb, object):
+class Pdb(OldPdb):
     """Modified Pdb class, does not load readline.
 
     for a standalone version that uses prompt_toolkit, see
@@ -227,10 +228,14 @@ class Pdb(OldPdb, object):
         self.shell = get_ipython()
 
         if self.shell is None:
+            save_main = sys.modules['__main__']
             # No IPython instance running, we must create one
             from IPython.terminal.interactiveshell import \
                 TerminalInteractiveShell
             self.shell = TerminalInteractiveShell.instance()
+            # needed by any code which calls __import__("__main__") after
+            # the debugger was entered. See also #9941.
+            sys.modules['__main__'] = save_main 
 
         if color_scheme is not None:
             warnings.warn(
@@ -278,37 +283,12 @@ class Pdb(OldPdb, object):
         """Shorthand access to the color table scheme selector method."""
         self.color_scheme_table.set_active_scheme(scheme)
 
-    def trace_dispatch(self, frame, event, arg):
-        try:
-            return super(Pdb, self).trace_dispatch(frame, event, arg)
-        except bdb.BdbQuit:
-            pass
-
     def interaction(self, frame, traceback):
         try:
             OldPdb.interaction(self, frame, traceback)
         except KeyboardInterrupt:
             sys.stdout.write('\n' + self.shell.get_exception_only())
 
-    def parseline(self, line):
-        if line.startswith("!!"):
-            # Force standard behavior.
-            return super(Pdb, self).parseline(line[2:])
-        # "Smart command mode" from pdb++: don't execute commands if a variable
-        # with the same name exists.
-        cmd, arg, newline = super(Pdb, self).parseline(line)
-        # Fix for #9611: Do not trigger smart command if the command is `exit`
-        # or `quit` and it would resolve to their *global* value (the
-        # `ExitAutocall` object).  Just checking that it is not present in the
-        # locals dict is not enough as locals and globals match at the
-        # toplevel.
-        if ((cmd in self.curframe.f_locals or cmd in self.curframe.f_globals)
-                and not (cmd in ["exit", "quit"]
-                         and (self.curframe.f_locals is self.curframe.f_globals
-                              or cmd not in self.curframe.f_locals))):
-            return super(Pdb, self).parseline("!" + line)
-        return super(Pdb, self).parseline(line)
-
     def new_do_up(self, arg):
         OldPdb.do_up(self, arg)
     do_u = do_up = decorate_fn_with_doc(new_do_up, OldPdb.do_up)
@@ -621,3 +601,12 @@ class Pdb(OldPdb, object):
                 self.print_stack_trace()
 
         do_w = do_where
+
+
+def set_trace(frame=None):
+    """
+    Start debugging from `frame`.
+
+    If frame is not specified, debugging starts from caller's frame.
+    """
+    Pdb().set_trace(frame or sys._getframe().f_back)
